@@ -1,4 +1,6 @@
 /*++++++++++++++
+ Copyright:    (C) 2008-2017 UDS/CNRS
+ License:       GNU General Public License
 .IDENTIFICATION wwwget.c
 .LANGUAGE       C
 .AUTHOR         Francois Ochsenbein [CDS]
@@ -48,6 +50,7 @@
 .VERSION  3.12  06-Jun-2012: Pb with -m option
 .VERSION  3.13  16-Aug-2012: Pb with content length
 .VERSION  3.14  05-Dec-2012: Read text to send from stdin if not in arguments
+.VERSION  3.15  25-Jun-2014: -A option
 .COMMENTS       This stand-alone program can send HTTP requests, and
 		displays the result on the standard output.
 	To compile:  cc wwwget.c -o wwwget  
@@ -110,7 +113,7 @@ The HTTP Statuses are:
    10.5.6   505 HTTP Version Not Supported ...........................71
 ---------------*/
 
-#define VERSION "3.13  (2012-08-15)"
+#define VERSION "3.14  (2014-06-25)"
 
 #define NODELAY	1
 #define isquote(c)	((c) == '"') || ((c) == '\'') || ((c) == '`')
@@ -173,6 +176,8 @@ Usage: wwwget [-v] [-s] [-m...] [-abs] [-post|-head|-get|-redirect] [-q]\n\
   -redirect: Generate a Redirect header (Status: 302 Moved Temporarily)\n\
    -nostrip: keep the header (with HTTP code, Content-Type, etc) in output\n\
   -c cookie: send the specified cookie\n\
+    -Aagent: specifies the user-agent (as User-agent: wwwget/agent)\n\
+             (can also be specified with the AGENT environment variable)\n\
    -Ddomain: domain to specify (in WWW-Authenticate)\n\
              (Domain is specified in WWW-Authenticate answer from server)\n\
      -Uuser: Userid to specify (in Authorization)\n\
@@ -215,6 +220,7 @@ static char *omoz ;	/* Emulate Mozilla */
 static char oq ;	/* -query Option    */
 static char *aD;	/* Domain */
 static char *aU;	/* UserID */
+static char *aA;	/* UserAgent */
 static char *aP ;	/* Passwd */
 static char *aR ;	/* Range  */
 static char *aF ;	/* From:  */
@@ -973,10 +979,9 @@ static void onIntr(int s)
 -----------------*/
   /* IN: s=Signal number */
 {
-  int st;
     if (ov) fprintf(stderr, " (INTR signal#%d)\n", s) ;
     if ((skp >= skbuf) && (skp < (skbuf+sizeof(skbuf))))
-        st = write(fileno(ofile), skbuf, skp - skbuf) ;
+        write(fileno(ofile), skbuf, skp - skbuf) ;
     fprintf(stderr, "\n****INTR exit\n") ;
     exit(1) ;
 }
@@ -988,12 +993,12 @@ static void tooLong(int s)
 -----------------*/
   /* IN: s=Signal number */
 {
-  int bytes, st;
+  int bytes;
     fprintf(stderr, "#***(ALARM signal (timeout=%ds)\n", reading_timeout) ;
     if ((skp >= skbuf) && (skp < (skbuf+sizeof(skbuf)))) {
 	bytes = skp - skbuf;
 	fprintf(stderr, "  [%d last bytes written out]\n", bytes);
-	st = write(fileno(ofile), skbuf, skp - skbuf) ;
+	write(fileno(ofile), skbuf, skp - skbuf) ;
     }
     fprintf(stderr, "\n****INTR exit\n") ;
     exit(1) ;
@@ -1002,7 +1007,7 @@ static void tooLong(int s)
 int main (int argc, char **argv)
 {
   char *p, *q, *host, *port, *body, *location, *qstring;
-  int from_arg, i, n, len7[7], len, path_len, http_status, sec, completed;
+  int from_arg, i, n, len7[7], len, path_len, http_status, st=0, sec, completed;
   static char sep[2];
   static char *method = "GET " ;
   static char *buf = (char *)0 ;
@@ -1087,6 +1092,10 @@ int main (int argc, char **argv)
 	    aF = p+2;
 	    if (aF) head_len += 10+strlen(aF);
 	    continue ;
+	  case 'A':	/* User-Agent */
+	    aA = p+2;
+	    if (aA) head_len += strlen(aA);
+	    continue;
 	  case 'U':	/* UserID */
 	    aU = p+2;
 	    if (aU) head_len += 40+strlen(aU);
@@ -1333,9 +1342,20 @@ int main (int argc, char **argv)
 	     }
 	     strcat(p, mozilla_def);
 	}
-	else sprintf(p+strlen(p), 
-	    "User-Agent: wwwget/%s\r\nAccept: */*\r\n", 
-	     VERSION);
+	else {
+	    strcat(p, "User-Agent: ");
+	    if (!aA) aA = getenv("AGENT");
+	    if (aA && *aA == ':') {
+		for(++aA; *aA==' '; aA++) ;
+		 strcat(p, aA);
+	    }
+	    else {
+		if(!aA) aA = VERSION;
+		strcat(p, "wwwget/");
+		strcat(p, aA);
+	    }
+	    strcat(p, "\r\nAccept: */*\r\n");
+	}
 #if 0
 	     strcat(p, "User-Agent: wwwget/2.3\r\nAccept: */*\r\n");
 #endif
@@ -1433,10 +1453,12 @@ int main (int argc, char **argv)
 		    for (location = skp+10; *location == ' '; location++) ;
 	    }
 	    if (i) { 		/* Is the End of the Header */
-		if (ostrip) { FILE *of ; int st;
+		if (ostrip) { FILE *of ;
 		    of = ostrip == 1 ? ofile : stderr ;
 		    fflush(of);
-		    st = write(fileno(of), skbuf, (skp+(i>>1)) - skbuf) ;
+		    if (write(fileno(of), skbuf, (skp+(i>>1)) - skbuf) <0) 
+		        fprintf(stderr, "#***write socket error: %s\n",
+			    strerror(errno));
 		    fprintf(of, "Host: %s%s (wwwget", hostname, url[URL_PORT]);
 		    if (oabs) fprintf(of, " -abs") ;
 		    fprintf(of, ")\r\nDocument: ") ;
@@ -1473,11 +1495,11 @@ int main (int argc, char **argv)
 	    continue;
 	}
 
-	if (ov && (skp>skbuf) && (!completed) && (!isatty(ofno))) { int st;
+	if (ov && (skp>skbuf) && (!completed) && (!isatty(ofno))) {
 	    fprintf(stderr, "#...Header%s is:\n", ostrip ? " (stripped)" : ""); 
 	    fflush(stderr) ;
 	    if (ostrip!=2) /* V2.4: don't rewrite a 2nd time... */
-		st = write(2, skbuf, skp-skbuf-1) ;
+		write(2, skbuf, skp-skbuf-1) ;
 	}
 
 	/* REDIRECT ... Do It ! */
@@ -1500,11 +1522,13 @@ int main (int argc, char **argv)
 	        else if (ostrip) fprintf(stderr, ": ");
 	        if (ostrip||ov)  fprintf(stderr, "%s\n", location);
 	    }
-	    if (redirect_max == 0) { int st;  /* V2.7: HEAD does not redirect */
+	    if (redirect_max == 0) { 	/* V2.7: HEAD does not redirect */
 		p = "Location: "; 
 		st = write(ofno, p, strlen(p));
-		st = write(ofno, location, strlen(location)) ;
-		st = write(ofno, "\r\n", 2);
+		if (st>=0) st = write(ofno, location, strlen(location)) ;
+		if (st>=0) st = write(ofno, "\r\n", 2);
+		if (st<0) fprintf(stderr, "#***Redirect error: %s\n",
+		     strerror(errno));
 		continue;
 	    }
 	    buf = malloc(n+22);
@@ -1551,7 +1575,7 @@ int main (int argc, char **argv)
 	}
 	else if (skbytes >= content_length)	/* Everything read... */
 	    completed = 1;
-	while(!completed /*skbytes < content_length*/) { int st;
+	while(!completed /*skbytes < content_length*/) {
 	    if (prompt && (promptno <= 0)) break;
 	    n = skread() ;
 	    if (n <= 0) break ;	/* Error */
@@ -1560,6 +1584,11 @@ int main (int argc, char **argv)
 	    if (oabs) exec_abs(ofile);
 	    else st = write(ofno, skbuf, len) ;  /* Error corrected, thanks to
 					            Aymeric SAUVAGEON */
+	    if (st<0) {
+		fprintf(stderr, "#**write buffer of %d bytes: %s\n", 
+			len, strerror(errno));
+		break;
+	    }
 	}
         if (ov) { double rate;
 	    sec = (time(0) - starting_time);
